unit frmMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, blcksock, ExtCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    btnSimpleTest: TButton;
    Timer1: TTimer;
    cbBroadcasting: TCheckBox;
    cbListening: TCheckBox;
    procedure btnSimpleTestClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure cbBroadcastingClick(Sender: TObject);
    procedure cbListeningClick(Sender: TObject);
  private
    FUDPSocket: TUDPBlockSocket;

    procedure MultiCastTest;

    procedure InitSocket;
    procedure DeInitSocket;
    { Private declarations }
  public
    { Public declarations }
  end;

  TUDPThread = class(TThread)
  private
    FUDPSocket: TUDPBlockSocket;
    FValue: string;
    procedure Log;
  public
    procedure Execute; override;
  end;

var
  Form1: TForm1;
  UDPThread: TUDPThread;

implementation

{$R *.dfm}

procedure TForm1.MultiCastTest;
var
  sndsock:TUDPBlockSocket;
  rcvsock:TUDPBlockSocket;
  buf:string;
begin
  sndsock:=TUDPBlockSocket.Create;
  rcvsock:=TUDPBlockSocket.Create;
  try
    rcvsock.createsocket;
    rcvsock.Bind('0.0.0.0','22401');
    rcvsock.AddMulticast('234.5.6.7');
    if rcvsock.LastError <> 0 then exit;

    sndsock.createsocket;
    sndsock.Bind('0.0.0.0','0');
    sndsock.MulticastTTL := 1;
    sndsock.connect('234.5.6.7','22401');
    if sndsock.LastError <> 0 then exit;

    sndsock.SendString('Test Payload');
    buf:=rcvsock.RecvPacket(1000);
    showmessage(buf);

    sndsock.CloseSocket;
    rcvsock.CloseSocket;
  finally
    sndsock.free;
    rcvsock.free;
  end;
end;

procedure TForm1.btnSimpleTestClick(Sender: TObject);
begin
  MultiCastTest;
end;

{ TUDPThread }

procedure TUDPThread.Execute;
begin
  FUDPSocket := TUDPBlockSocket.Create();

  FUDPSocket.CreateSocket;
  FUDPSocket.Bind('0.0.0.0', '22401');
  FUDPSocket.AddMulticast('234.5.6.7');
  if FUDPSocket.LastError <> 0 then Exit;

  while not Terminated do begin
    FValue := FUDPSocket.RecvString(1000);
    if FUDPSocket.LastError = 0 then
      Synchronize(Log);
  end;

  FUDPSocket.Free;
end;

procedure TUDPThread.Log;
begin
  //not safe but functional
  Form1.Memo1.Lines.Add('<' + FValue);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  DeInitSocket;
  if UDPThread <> nil then begin
    UDPThread.Terminate;
    UDPThread.WaitFor;
  end;
end;

var
  LineCount : Longint = 0;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  MsgStr: string;
begin
  InitSocket;
  if FUDPSocket = nil then Exit;

  Inc(LineCount);
  if LineCount mod 10 = 0 then Memo1.Lines.Clear;

  MsgStr := IntToStr(LineCount) + ':' + DateTimeToStr(Now);
  FUDPSocket.SendString(MsgStr + #13#10);
  Memo1.Lines.Add('>' + MsgStr);
  DeInitSocket;
end;

procedure TForm1.cbBroadcastingClick(Sender: TObject);
begin
  Timer1.Enabled := cbBroadcasting.Checked;
end;

procedure TForm1.cbListeningClick(Sender: TObject);
begin
  if cbListening.Checked then begin
    UDPThread := TUDPThread.Create(False);
  end else begin
    if UDPThread <> nil then begin
      UDPThread.Terminate;
      UDPThread.WaitFor;
      UDPThread := nil;
    end;
  end;
end;

procedure TForm1.DeInitSocket;
begin
  FUDPSocket.Free;
  FUDPSocket := nil;
end;

procedure TForm1.InitSocket;
begin
  if FUDPSocket <> nil then
    DeInitSocket;

  FUDPSocket := TUDPBlockSocket.Create;
  FUDPSocket.CreateSocket;
  FUDPSocket.Bind('0.0.0.0', '0');
  FUDPSocket.MulticastTTL := 1;
  FUDPSocket.Connect('234.5.6.7', '22401');
end;

end.
